/**
 * SortedList
 *
 * author : shinout <shinout310@gmail.com>
 *
 * example: 
 *
 *  var SortedList = require("sortedlist");
 *
 *  var list = new SortedList({
 *    [152, 222], [33, 53], [48, 96], [928, 1743], [66, 67], [11, 12]
 *  });
 *
 *  // insert new value 
 *  list.insert([193, 202]); 
 *
 *
 *  // get value by position 
 *  var val = list.get(3); 
 *
 *
 *  // get the first value 
 *  var first = list.head(); 
 *
 *
 *  // get the last value 
 *  var last = list.tail(); 
 *
 *
 *  // set sort rule
 *  last.compare  = function(a, b) {
 *    return (a[0] > b[0]) ? 1 : -1;
 *  };
 *  // we can set sort rule in constructor as
 *  // new SortedList(arr, {compare: function(a,b){ // here } });
 *
 *
 *  // remove by position
 *  list.remove(4); 
 *
 *
 *  // get array
 *  var arr = list.toArray(); 
 *
 **/
function SortedList(arr, options) {
  if (arr instanceof Array) {
    return SortedList.build(arr, options);
  }
  this.arr = [];
  options = options || {};
  Object.keys(options).forEach(function(k) {
    this[k] = options[k];
  }, this);
}

SortedList.build = function(arr, options) {
  var list = new SortedList(null, options);
  arr.forEach(function(v) {
    list.insert(v);
  });
  return list;
};

SortedList.prototype.bsearch = function(val) {
  var mpos,
      spos = 0,
      epos = this.arr.length;
  while (epos - spos > 1) {
    mpos = Math.floor((spos + epos)/2);
    mval = this.arr[mpos];
    switch (this.compare(val, mval)) {
    case 1  :
    default :
      spos = mpos;
      break;
    case -1 :
      epos = mpos;
      break;
    case 0  :
      return mpos;
    }
  }
  return (this.arr[0] == null || spos == 0 && this.arr[0] != null && this.compare(this.arr[0], val) == 1) ? -1 : spos;
};

SortedList.prototype.get = function(pos) {
  return this.arr[pos];
};

SortedList.prototype.toArray = function(pos) {
  return this.arr.slice();
};

SortedList.prototype.size = function() {
  return this.arr.length;
};

SortedList.prototype.head = function() {
  return this.arr[0];
};

SortedList.prototype.tail = function() {
  return (this.arr.length == 0) ? null : this.arr[this.arr.length -1];
};

SortedList.prototype.insert = function(val) {
  var pos = this.bsearch(val);
  if (this.filter(val, pos)) {
    this.arr.splice(pos+1, 0, val);
  }
};

SortedList.prototype.filter = function(val, pos) {
  return true;
};

SortedList.prototype.add = SortedList.prototype.insert;

SortedList.prototype.delete = function(pos) {
  this.arr.splice(pos, 1);
};
SortedList.prototype.remove = SortedList.prototype.delete;

SortedList.prototype.compare = function(a, b) {
  var c = a - b;
  return (c > 0) ? 1 : (c == 0)  ? 0 : -1;
};


module.exports = SortedList;
